(* $Id: markup_codewriter.ml,v 1.1.2.1 2000/05/14 14:42:49 gerd Exp $
 * ----------------------------------------------------------------------
 *
 *)

open Markup_document
open Markup_yacc
open Markup_dtd
open Markup_types

let write_expr_ext_id out extid =
  match extid with
      System s ->
	output_string out ("(Markup_types.System\"" ^ String.escaped s ^ "\")")
    | Public(s,t) ->
	output_string out ("(Markup_types.Public(\"" ^ String.escaped s ^ 
			   "\",\"" ^
			   String.escaped t ^ "\"))")
;;


let rec write_expr_content_model out cm =
  match cm with
      Unspecified -> output_string out "Markup_types.Unspecified"
    | Empty       -> output_string out "Markup_types.Empty"
    | Any         -> output_string out "Markup_types.Any"
    | Mixed msl   -> output_string out "(Markup_types.Mixed [";
	             List.iter
	               (fun ms ->
			  write_expr_mixed_spec out ms;
			  output_string out "; ";
		       )
		       msl;
		     output_string out "])";
    | Regexp re   -> output_string out "(Markup_types.Regexp ";
	             write_expr_regexp_spec out re;
		     output_string out ")";

and write_expr_mixed_spec out ms =
  match ms with
      MPCDATA  -> output_string out "Markup_types.MPCDATA"
    | MChild s -> output_string out ("(Markup_types.MChild \"" ^
				     String.escaped s ^ "\")")

and write_expr_regexp_spec out re =
  match re with
      Optional re'  -> output_string out "(Markup_types.Optional ";
	               write_expr_regexp_spec out re';
		       output_string out ")";
    | Repeated re'  -> output_string out "(Markup_types.Repeated ";
	               write_expr_regexp_spec out re';
		       output_string out ")";
    | Repeated1 re' -> output_string out "(Markup_types.Repeated1 ";
	               write_expr_regexp_spec out re';
		       output_string out ")";
    | Alt rel       -> output_string out "(Markup_types.Alt [";
	               List.iter
			 (fun re' ->
			    write_expr_regexp_spec out re';
			    output_string out "; ";
			 )
			 rel;
		       output_string out "])";
    | Seq rel       -> output_string out "(Markup_types.Seq [";
	               List.iter
			 (fun re' ->
			    write_expr_regexp_spec out re';
			    output_string out "; ";
			 )
			 rel;
		       output_string out "])";
    | Child s       -> output_string out ("(Markup_types.Child \"" ^ 
					  String.escaped s ^ "\")")
;;


let write_expr_att_type out at =
  match at with
      A_cdata       -> output_string out "Markup_types.A_cdata"
    | A_id          -> output_string out "Markup_types.A_id"
    | A_idref       -> output_string out "Markup_types.A_idref"
    | A_idrefs      -> output_string out "Markup_types.A_idrefs"
    | A_entity      -> output_string out "Markup_types.A_entity"
    | A_entities    -> output_string out "Markup_types.A_entities"
    | A_nmtoken     -> output_string out "Markup_types.A_nmtoken"
    | A_nmtokens    -> output_string out "Markup_types.A_nmtokens"
    | A_notation sl -> output_string out "(Markup_types.A_notation [";
	               List.iter
			 (fun s ->
			    output_string out ("\"" ^ 
					       String.escaped s ^ "\"; "))
			 sl;
		       output_string out "])";
    | A_enum sl     -> output_string out "(Markup_types.A_enum [";
	               List.iter
			 (fun s ->
			    output_string out ("\"" ^ 
					       String.escaped s ^ "\"; "))
			 sl;
		       output_string out "])";
;;


let write_expr_att_default out ad =
  match ad with
      D_required  -> output_string out "Markup_types.D_required"
    | D_implied   -> output_string out "Markup_types.D_implied"
    | D_default s -> output_string out ("(Markup_types.D_default \"" ^
					String.escaped s ^ "\")")
    | D_fixed s   -> output_string out ("(Markup_types.D_fixed \"" ^
					String.escaped s ^ "\")")
;;


let write_expr_att_value out av =
  match av with
      Value s       -> output_string out ("(Markup_types.Value \"" ^
					  String.escaped s ^ "\")")
    | Valuelist sl  -> output_string out ("(Markup_types.Valuelist [");
	               List.iter
			 (fun s ->
			    output_string out ("\"" ^ String.escaped s ^ 
					       "\"; ")
			 )
			 sl;
		       output_string out "])";
    | Implied_value -> output_string out "Markup_types.Implied_value"
;;


let write_expr_new_pi out pi =
  output_string out ("(new Markup_dtd.proc_instruction \"" ^
		     String.escaped(pi # target) ^ "\" \"" ^
		     String.escaped(pi # value) ^ "\")")
;;


let write_expr_node_type out nt =
  match nt with
      T_data      -> output_string out "Markup_document.T_data"
    | T_element s -> output_string out ("(Markup_document.T_element \"" ^
					String.escaped s ^ "\")")
;;


let write_local_dtd out (dtd : dtd) =
  (* Outputs "let mkdtd warner = ... in" to 'out' *)
  output_string out "let mkdtd warner =\n";
  output_string out "let dtdobj = new Markup_dtd.dtd warner in\n";
  
  (* Set the ID: *)
  output_string out "dtdobj # set_id ";
  begin match dtd # id with
      None -> ()
    | Some(External x) -> 
	output_string out "(Markup_types.External ";
	write_expr_ext_id out x;
	output_string out ");\n"
    | Some(Derived x) ->  
	output_string out "(Markup_types.Derived ";
	write_expr_ext_id out x;
	output_string out ");\n"
    | Some Internal ->   
	output_string out "Markup_types.Internal;\n";
  end;

  (* Add elements: *)
  List.iter
    (fun elname ->
       (* Create the element 'el': *)
       let el = dtd # element elname in
       output_string out ("let el = new Markup_dtd.dtd_element dtdobj \"" ^
			  String.escaped elname ^ "\" in\n");
       output_string out "let cm = ";
       write_expr_content_model out (el # content_model);
       output_string out " in\n";
       output_string out "el # set_content_model cm;\n";
       (* Add attributes: *)
       List.iter
	 (fun attname ->
	    let atttype, attdefault = el # attribute attname in
	    output_string out ("el # add_attribute \"" ^ 
			       String.escaped attname ^ "\" ");
	    write_expr_att_type out atttype;
	    output_string out " ";
	    write_expr_att_default out attdefault;
	    output_string out ";\n";
	 )
	 (el # attribute_names);

       (* Validate: *)
       output_string out "el # validate;\n";
 
       (* Add the element 'el' to 'dtdobj': *)
       output_string out "dtdobj # add_element el;\n";
    )
    (dtd # element_names);

  (* Add notations: *)
  List.iter
    (fun noname ->
       let no = dtd # notation noname in
       output_string out ("let no = new Markup_dtd.dtd_notation \"" ^
			  String.escaped noname ^ "\" ");
       write_expr_ext_id out (no # ext_id);
       output_string out " in\n";
       output_string out "dtdobj # add_notation no;\n";
    )
    (dtd # notation_names);

  (* Add processing instructions: *)
  List.iter
    (fun target ->
       let pilist = dtd # pinstr target in
       List.iter
	 (fun pi ->
	    output_string out "let pi = ";
	    write_expr_new_pi out pi;
	    output_string out " in\n";
	    output_string out "dtdobj # add_pinstr pi;\n";
	 )
	 pilist;
    )
    (dtd # pinstr_names);

  (* Set the name of the root element: *)
  begin match dtd # root with
      None -> ()
    | Some rootname ->
	output_string out ("dtdobj # set_root \"" ^
			   String.escaped rootname ^ "\";\n")
  end;

  (* Special options: *)
  if dtd # arbitrary_allowed then
    output_string out "dtdobj # allow_arbitrary;\n"
  else
    output_string out "dtdobj # disallow_arbitrary;\n";

  (* Return dtdobj: *)
  output_string out "dtdobj in\n"
;;


let rec write_local_subtree out n =
  (* Outputs the term generating the subtree *)
  
  output_string out "let nt = ";
  write_expr_node_type out (n # node_type);
  output_string out " in\n";
  begin match n # node_type with
      T_data ->
	output_string out ("let t = mk_data map dtd \"" ^ 
			   String.escaped (n # data) ^ "\" in\n")
    | T_element _ ->
	output_string out "let t = mk_element map dtd nt [\n";
	List.iter
	  (fun (name,value) ->
	     begin match value with
		 Value s -> 
		   output_string out ("\"" ^ String.escaped name ^ "\", ");
		   output_string out ("\"" ^ String.escaped s ^ "\"; ")
	       | Valuelist sl ->
		   output_string out ("\"" ^ String.escaped name ^ "\", ");
		   output_string out ("\"" ^ 
				      String.escaped (String.concat " " sl) ^ 
				      "\"; ")
	       | Implied_value ->
		   ()
	     end
	  )
	  (n # attributes);
	output_string out " ] in\n";
  end;

  (* Add processing instructions: *)
  List.iter
    (fun target ->
       let pilist = n # pinstr target in
       List.iter
	 (fun pi ->
	    output_string out "let pi = ";
	    write_expr_new_pi out pi;
	    output_string out " in\n";
	    output_string out "add_pinstr t pi;\n";
	 )
	 pilist;
    )
    (n # pinstr_names);
       
  (* Add the sub nodes: *)
  n # iter_nodes
    (fun n' ->
       output_string out "add_node t (\n";
       write_local_subtree out n';
       output_string out ");\n";
    );

  (* Validate: *)
  output_string out "local_validate t;\n";

  (* Return: *)
  output_string out "t\n"
;;


let write_local_document out (d : 'ext document) =
  (* Outputs "let mkdoc warner map = ... in" *)
  
  output_string out "let mkdoc warner map =\n";
  output_string out "let doc = new Markup_document.document warner in\n";
  output_string out ("doc # init_xml_version \"" ^
		     String.escaped (d # xml_version) ^ "\";\n");
  output_string out ("doc # init_xml_standalone " ^
		     (if d # xml_standalone then "true" else "false") ^
		     ";\n");
  write_local_dtd out (d # dtd);
  output_string out "let dtd = mkdtd warner in\n";
  output_string out "let root = ";
  write_local_subtree out (d # root);
  output_string out "in\n";
  output_string out "doc # init_root root;\n";

  (* Add processing instructions: *)
  List.iter
    (fun target ->
       let pilist = d # pinstr target in
       List.iter
	 (fun pi ->
	    output_string out "let pi = ";
	    write_expr_new_pi out pi;
	    output_string out " in\n";
	    output_string out "doc # add_pinstr pi;\n";
	 )
	 pilist;
    )
    (d # pinstr_names);
  
  (* Return the result: *)
  output_string out "doc in\n"
;;


let write_helpers out =
  output_string out "let find_exemplar map nt = try Hashtbl.find map.Markup_yacc.map nt with Not_found -> map.Markup_yacc.default_element in\n";
  output_string out "let mk_data map dtd s = ((find_exemplar map Markup_document.T_data) # create_data dtd s : 'ext Markup_document.node) in\n";
  output_string out "let mk_element map dtd nt atts = ((find_exemplar map nt) # create_element dtd nt atts : 'ext Markup_document.node) in\n";
  output_string out "let add_node t n = (t : 'ext Markup_document.node) # add_node (n : 'ext Markup_document.node) in\n";
  output_string out "let add_pinstr t pi = (t : 'ext Markup_document.node) # add_pinstr (pi : Markup_dtd.proc_instruction) in\n";
  output_string out "let local_validate t = (t : 'ext Markup_document.node) # local_validate in\n"
;;

 
let write_document out d =
  output_string out "let create_document warner map =\n";
  write_helpers out;
  write_local_document out d;
  output_string out "mkdoc warner map;;\n"
;;


let write_dtd out dtd =
  output_string out "let create_dtd warner =\n";
  write_local_dtd out dtd;
  output_string out "mkdtd warner;;\n"
;;


let write_subtree out t =
  output_string out "let create_subtree dtd map =\n";
  write_helpers out;
  write_local_subtree out t;
  output_string out ";;\n"
;;

(* ======================================================================
 * History:
 * 
 * $Log: markup_codewriter.ml,v $
 * Revision 1.1.2.1  2000/05/14 14:42:49  gerd
 * 	Improved: The resulting code can be compiled faster, and the
 * compiler is less hungry on memory.
 *
 * Revision 1.1  2000/03/11 22:57:28  gerd
 * 	Initial revision.
 *
 * 
 *)
