(* $Id: markup_lexer_types_shadow.mli,v 1.3 1999/08/31 19:13:31 gerd Exp $
 * ----------------------------------------------------------------------
 * Markup! The validating XML parser for Objective Caml.
 * Copyright 1999 by Gerd Stolpmann. See LICENSE for details.
 *)

type lexers =
    Document
  | Document_type
  | Content
  | Within_tag
  | Declaration
  | Ignored_section


type prolog_token =
    Pro_name of string
  | Pro_eq                  (* "=" *)
  | Pro_string of string    (* "..." or '...' *)
  | Pro_eof

type entity_id = < >
  (* The class without properties; but you can still compare if two objects
   * are the same.
   *)

(* -- The actual definition of 'token' will be appended to
 * markup_lexer_types_shadow.mli and be written to markup_lexer_types.mli.
 * The following is an earlier version of 'token' that was commented.
 *)
(*
type token =
    Begin_entity             (* Beginning of entity *)
  | End_entity               (* End of entity *)
  | PI of (string * string)  (* <?name ... ?> *)
  | PI_xml of prolog_token list (* <?xml ...?> *)
  | Doctype                  (* <!DOCTYPE *)
  | Doctype_rangle           (* > as DOCTYPE delimiter *)
  | Comment                  (* <!-- ... --> *)
  | Tag_beg of string        (* <name *)
  | Tag_end of string        (* </name *)
  | Rangle                   (* > as tag delimiter *)
  | Rangle_empty             (* /> as tag delimiter *)
  | Cdata of string          (* <![CDATA[...]]> *)
  | CRef of int              (* &#digits; *)
  | ERef of string           (* &name; *)
  | PERef of string          (* %name; *)
  | CharData of string       (* any characters not otherwise matching *)
  | Ignore                   (* ignored whitespace *)
  | Name of string           (* name *)
  | Nametoken of string      (* nmtoken but not name *)
  | Eq                       (* = *)
  | Attval of string         (* attribute value; may contain entity refs *)
  | Unparsed_string of string  (* "data" or 'data' *)
  | Dtd_begin                (* '[' after DOCTYPE *)
  | Dtd_end                  (* ']' *)
  | Conditional_begin        (* <![ in declaration *)
  | Conditional_body         (* [ in declaration *)
  | Conditional_end          (* ]]> in declaration *)
  | Percent                  (* % followed by space in declaration *)
  | Lparen                   (* ( in declaration *)
  | Rparen                   (* ) in declaration *)
  | Plus                     (* + in declaration *)
  | Star                     (* * in declaration *)
  | Bar                      (* | in declaration *)
  | Comma                    (* , in declaration *)
  | Qmark                    (* ? in declaration *)
  | RparenPlus               (* )+ in declaration *)
  | RparenStar               (* )* in declaration *)
  | RparenQmark              (* )? in declaration *)  
  | Pcdata                   (* #PCDATA in declaration *)
  | Required                 (* #REQUIRED in declaration *)
  | Implied                  (* #IMPLIED in declaration *)
  | Fixed                    (* #FIXED in declaration *)
  | Decl_element             (* <!ELEMENT *)
  | Decl_attlist             (* <!ATTLIST *)
  | Decl_entity              (* <!ENTITY *)
  | Decl_notation            (* <!NOTATION *)
  | Decl_rangle              (* > *)
  | Eof
*)

(* INCLUDE_HERE *)

val string_of_tok : token -> string

(* ======================================================================
 * History:
 * 
 * $Log: markup_lexer_types_shadow.mli,v $
 * Revision 1.3  1999/08/31 19:13:31  gerd
 * 	Added checks on proper PE nesting. The idea is that tokens such
 * as Decl_element and Decl_rangle carry an entity ID with them. This ID
 * is simply an object of type < >, i.e. you can only test on identity.
 * The lexer always produces tokens with a dummy ID because it does not
 * know which entity is the current one. The entity layer replaces the dummy
 * ID with the actual ID. The parser checks that the IDs of pairs such as
 * Decl_element and Decl_rangle are the same; otherwise a Validation_error
 * is produced.
 *
 * Revision 1.2  1999/08/10 21:35:09  gerd
 * 	The XML/encoding declaration at the beginning of entities is
 * evaluated. In particular, entities have now a method "xml_declaration"
 * which returns the name/value pairs of such a declaration. The "encoding"
 * setting is interpreted by the entity itself; "version", and "standalone"
 * are interpreted by Markup_yacc.parse_document_entity. Other settings
 * are ignored (this does not conform to the standard; the standard prescribes
 * that "version" MUST be given in the declaration of document; "standalone"
 * and "encoding" CAN be declared; no other settings are allowed).
 * 	TODO: The user should be warned if the standard is not exactly
 * fulfilled. -- The "standalone" property is not checked yet.
 *
 * Revision 1.1  1999/08/10 00:35:51  gerd
 * 	Initial revision.
 *
 * 
 *)
